Imports System.Math
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.Runtime.InteropServices

<ToolboxBitmap(GetType(HsColorWheel), "tbxHsColorWheel")> _
<DefaultEvent("ColorScrolled")> _
Public Class HsColorWheel
    Inherits System.Windows.Forms.Control

    ' The width of the little box that shows the selected color.
    Private Const BOX_RADIUS As Integer = 2
    Private Const BOX_WID As Integer = 2 * BOX_RADIUS + 1

    ' Drawing geometry.
    Private m_Radius As Integer
    Private m_Cx As Single
    Private m_Cy As Single

    ' Raised when the user moves the mouse with the left button down.
    Public Event ColorScrolled(ByVal hue As Single, ByVal saturation As Single)

    ' Raised when the user releases the mouse.
    Public Event ColorPicked(ByVal hue As Single, ByVal saturation As Single)

    ' The color wheel's image.
    Private m_Bitmap As Bitmap

    ' The currently selected color.
    ' Hue is stored as an angle in radians.
    Private m_Hue As Single = 0
    Public Property Hue() As Single
        Get
            Dim return_value As Single = CSng(m_Hue / PI * 180)
            If return_value < 0 Then return_value += 360
            If return_value <= 1 Then return_value = 360
            Return 360 - return_value
        End Get
        Set(ByVal Value As Single)
            m_Hue = CSng((360 - Value) / 180 * PI)
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' Saturation is stored as a value between 0 and 1.
    Private m_Saturation As Single = 1
    Public Property Saturation() As Single
        Get
            Return m_Saturation
        End Get
        Set(ByVal Value As Single)
            If Value < 0 Then Value = 0
            If Value > 1 Then Value = 1
            m_Saturation = Value
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' The currently selected color with a given lightness.
    Public Property SelectedColor(Optional ByVal use_lightness As Single = 0.5) As Color
        Get
            Dim r, g, b As Double
            ColorConversions.HlsToRgb(m_Hue, use_lightness, m_Saturation, r, g, b)
            Return Color.FromArgb(255, CInt(r), CInt(g), CInt(b))
        End Get
        Set(ByVal Value As Color)
            Dim r, g, b, h, l, s As Double
            r = Value.R
            g = Value.G
            b = Value.B
            ColorConversions.RgbToHls(r, g, b, h, l, s)
            m_Hue = CSng((360 - h) / 180 * PI)
            m_Saturation = CSng(s)
            Me.Invalidate()
        End Set
    End Property

    ' Create the control's background image.
    Private Sub HsColorWheel_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' Use double-buffering to prevent flicker.
        Me.SetStyle( _
            ControlStyles.AllPaintingInWmPaint Or _
            ControlStyles.UserPaint Or _
            ControlStyles.DoubleBuffer, _
            True)
        Me.UpdateStyles()

        ' Make a path.
        Dim diameter As Single = Min(Me.Width, Me.Height) - BOX_WID - 2
        m_Radius = CInt(diameter / 2)
        m_Cx = CInt(Me.Width / 2)
        m_Cy = CInt(Me.Height / 2)
        Dim rect As New RectangleF(m_Cx - m_Radius, m_Cy - m_Radius, m_Radius * 2, m_Radius * 2)
        Dim path As New GraphicsPath
        path.AddEllipse(rect)
        path.Flatten()

        ' Make the colors for the path's points.
        Dim num_pts As Integer = (path.PointCount - 1) \ 3
        Dim surround_colors(path.PointCount - 1) As Color
        Dim r, g, b, angle As Double
        For i As Integer = 0 To path.PointCount - 1
            angle = -Atan2(path.PathPoints(i).Y - m_Cy, path.PathPoints(i).X - m_Cx) * 180 / PI
            ColorPickerControls.ColorConversions.HlsToRgb(angle, 0.5, 1, r, g, b)
            surround_colors(i) = Color.FromArgb(255, CInt(r), CInt(g), CInt(b))
        Next i

        ' Make a brush for the path.
        Dim br As New PathGradientBrush(path)
        br.CenterColor = Color.FromArgb(255, 128, 128, 128)
        br.SurroundColors = surround_colors

        ' Draw onto a Bitmap.
        m_Bitmap = New Bitmap(Me.Width, Me.Height)
        Dim gr As Graphics = Graphics.FromImage(m_Bitmap)
        gr.Clear(Me.BackColor)

        gr.FillPath(br, path)
        Dim outline_pen As New Pen(Me.ForeColor, 0)
        gr.DrawPath(outline_pen, path)
        outline_pen.Dispose()

        ' Uncomment the following to draw the path's points.
        'For i As Integer = 0 To path.PointCount - 1
        '    gr.DrawEllipse(Pens.Red, _
        '        path.PathPoints(i).X - 1, _
        '        path.PathPoints(i).Y - 1, _
        '        2, 2)
        'Next i

        gr.Dispose()

        ' Force a redraw.
        Me.Invalidate()
    End Sub

    ' Redraw.
    Private Sub HsColorWheel_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        ' Draw the color wheel.
        e.Graphics.DrawImage(m_Bitmap, 0, 0)

        ' Convert Hue and Saturation into X and Y.
        Dim radius As Single = m_Saturation * m_Radius
        Dim x As Single = CSng(m_Cx + radius * Cos(m_Hue))
        Dim y As Single = CSng(m_Cy + radius * Sin(m_Hue))

        ' Draw the color selection box.
        e.Graphics.DrawRectangle(Pens.Black, _
            x - BOX_RADIUS, y - BOX_RADIUS, _
            BOX_WID, BOX_WID)
    End Sub

    ' If the left button is down, select the color and raise ColorScrolled.
    Private Sub HsColorWheel_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove, MyBase.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SavePoint(e.X, e.Y)
        End If
    End Sub

    ' If this point lies within the control's region, save it.
    Private Sub SavePoint(ByVal x As Integer, ByVal y As Integer)
        ' Get the point's angle and distance.
        Dim cx As Single = CSng(Me.Width / 2)
        Dim cy As Single = CSng(Me.Height / 2)
        Dim dx As Single = x - cx
        Dim dy As Single = y - cy
        Dim dist As Single = CSng(Sqrt(dx * dx + dy * dy))
        Dim angle As Single = CSng(Atan2(dy, dx))

        ' Make sure the point is on the wheel.
        If dist > m_Radius Then
            dist = m_Radius
            x = CInt(cx + m_Radius * Cos(angle))
            y = CInt(cy + m_Radius * Sin(angle))
        End If

        ' Save the point.
        m_Hue = angle
        m_Saturation = dist / m_Radius

        ' Redraw.
        Me.Invalidate()

        ' Raise the ColorScrolled event.
        RaiseEvent ColorScrolled(Hue, Saturation)
    End Sub

    ' Raise the ColorPicked event.
    Private Sub HsColorWheel_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        RaiseEvent ColorPicked(Hue, Saturation)
    End Sub
End Class
