Imports System.Math
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.Runtime.InteropServices

<ToolboxBitmap(GetType(LightnessBar), "tbxLightnessBar")> _
<DefaultEvent("LightnessScrolled")> _
Public Class LightnessBar
    Inherits System.Windows.Forms.Control

    ' Raised when the user moves the mouse with the left button down.
    Public Event LightnessScrolled(ByVal new_lightness As Single)

    ' Raised when the user releases the mouse.
    Public Event LightnessPicked(ByVal new_lightness As Single)

    ' The width of the triangle that shows the selected lightness.
    Private Const TRI_WID As Integer = 5
    Private Const TRI_MARGIN As Integer = 1

    ' Lightness is between 0 (black) and 1 (white).
    Private m_Lightness As Single = 0.5
    Public Property Lightness() As Single
        Get
            Return m_Lightness
        End Get
        Set(ByVal Value As Single)
            m_Lightness = Value
            If m_Lightness < 0 Then m_Lightness = 0
            If m_Lightness > 1 Then m_Lightness = 1
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' The basic color.
    Private m_BaseColor As Color = Color.Blue
    Public Property BaseColor() As Color
        Get
            Return m_BaseColor
        End Get
        Set(ByVal Value As Color)
            m_BaseColor = Value
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' The currently selected color.
    Private m_SelectedColor As Color = Color.Blue
    Public Property SelectedColor() As Color
        Get
            ' Convert RGB to HLS.
            Dim r, g, b, h, l, s As Double
            r = m_BaseColor.R
            g = m_BaseColor.G
            b = m_BaseColor.B
            ColorConversions.RgbToHls(r, g, b, h, l, s)

            ' Convert to RGB using the selected lightness.
            ColorConversions.HlsToRgb(h, m_Lightness, s, r, g, b)

            Return Color.FromArgb(255, CInt(r), CInt(g), CInt(b))
        End Get
        Set(ByVal Value As Color)
            ' Convert RGB to HLS.
            Dim r, g, b, h, l, s As Double
            r = Value.R
            g = Value.G
            b = Value.B
            ColorConversions.RgbToHls(r, g, b, h, l, s)

            ' Convet back to RGB with lightness = 0.5.
            ColorConversions.HlsToRgb(h, 0.5, s, r, g, b)

            ' Use this as the base color.
            m_BaseColor = Color.FromArgb(255, CInt(r), CInt(g), CInt(b))

            ' Set our Lightness value.
            m_Lightness = CSng(l)

            ' Redraw.
            Me.Invalidate()
        End Set
    End Property

    ' Force a redraw.
    Private Sub LightnessBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' Use double-buffering to prevent flicker.
        Me.SetStyle( _
            ControlStyles.AllPaintingInWmPaint Or _
            ControlStyles.UserPaint Or _
            ControlStyles.DoubleBuffer, _
            True)
        Me.UpdateStyles()

        ' Redraw.
        Me.Invalidate()
    End Sub

    ' Redraw.
    Private Sub LightnessBar_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        e.Graphics.Clear(Me.BackColor)

        ' Draw the top half: White to BaseColor.
        Dim wid As Integer = Me.Width - TRI_MARGIN - TRI_WID
        Dim half_hgt As Integer = Me.Height \ 2
        Dim rect As New RectangleF(0, 0, wid, half_hgt)
        Dim br As LinearGradientBrush
        br = New LinearGradientBrush(rect, _
            Color.White, _
            m_BaseColor, _
            LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(br, rect)
        br.Dispose()

        ' Draw the bottom half: BaseColor to Black.
        rect.Y += half_hgt - 1
        rect.Height = Me.Height - half_hgt + 1
        br = New LinearGradientBrush(rect, _
            m_BaseColor, _
            Color.Black, _
            LinearGradientMode.Vertical)
        rect.Y += 1
        e.Graphics.FillRectangle(br, rect)
        br.Dispose()

        ' Draw the triangle.
        Dim x As Integer = Me.Width - TRI_WID
        Dim y As Integer = CInt(Me.Height * (1 - m_Lightness))
        Dim pts() As Point = { _
            New Point(x, y), _
            New Point(x + TRI_WID, y - TRI_WID), _
            New Point(x + TRI_WID, y + TRI_WID), _
            New Point(x, y) _
        }
        Dim fore_brush As New SolidBrush(Me.ForeColor)
        e.Graphics.FillPolygon(fore_brush, pts)
        fore_brush.Dispose()
    End Sub

    Private Sub LightnessBar_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove, MyBase.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SavePoint(e.Y)
        End If
    End Sub

    ' Save this point.
    Private Sub SavePoint(ByVal y As Integer)
        ' Get the corresponding Lightness value.
        m_Lightness = CSng((Me.Height - y) / Me.Height)
        If m_Lightness < 0 Then m_Lightness = 0
        If m_Lightness > 1 Then m_Lightness = 1

        ' Redraw.
        Me.Invalidate()

        ' Raise the LightnessScrolled event.
        RaiseEvent LightnessScrolled(Lightness)
    End Sub

    ' Raise the LightnessPicked event.
    Private Sub HsColorWheel_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        RaiseEvent LightnessPicked(Lightness)
    End Sub
End Class
