Imports System.ComponentModel
Imports System.Drawing.Drawing2D

<DefaultProperty("MapImage")> _
<DefaultEvent("ThumbScrollComplete")> _
Public Class MapViewFinder

#Region "Public Events"
    Public Event ThumbScroll()
    Public Event ThumbScrollComplete()
#End Region ' Public Events

#Region "Public Properties"
    ' The full-scale map image.
    Private m_MapImage As Image = Nothing
    <Category("Appearance")> _
    <Browsable(True)> _
    Public Property MapImage() As Image
        Get
            Return m_MapImage
        End Get
        Set(ByVal value As Image)
            m_MapImage = value

            ' Make a new background image.
            MakeTransformation()
        End Set
    End Property

    ' The size of the area on the full-scale map that should be visible.
    Private m_ThumbLocation As New Rectangle(0, 0, 0, 0)
    <Category("Appearance")> _
    <Browsable(True)> _
    Public Property ThumbLocation() As Rectangle
        Get
            Return m_ThumbLocation
        End Get
        Set(ByVal value As Rectangle)
            m_ThumbLocation = value

            ' Redraw.
            Me.Invalidate()
        End Set
    End Property
#End Region ' Public Properties

#Region "Private Variables"
    ' The transformation we draw with.
    Private m_Transformation As Matrix = Nothing

    ' The inverse transformation.
    Private m_InverseTransformation As Matrix = Nothing

    ' Variables for dragging the thumb.
    Private m_Dragging As Boolean = False
    Private m_PrevX As Integer
    Private m_PrevY As Integer
#End Region ' Private Variables

    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)

        'Add your custom paint code here
        DrawControl(pe.Graphics)
    End Sub

    ' Draw the control.
    Private Sub DrawControl(ByVal gr As Graphics)
        ' Do nothing if we have no image or transformation.
        If (m_MapImage Is Nothing) Then
            gr.DrawLine(Pens.Blue, 0, 0, Me.Width, Me.Height)
            gr.DrawLine(Pens.Blue, 0, Me.Height, Me.Width, 0)
            Exit Sub
        End If

        ' Set the transformation.
        gr.Transform = m_Transformation

        ' Draw the image.
        gr.InterpolationMode = Drawing2D.InterpolationMode.High
        gr.DrawImage(m_MapImage, 0, 0, m_MapImage.Width, m_MapImage.Height)

        ' gr.DrawRectangle(Pens.Red, 10, 10, m_MapImage.Width - 20, m_MapImage.Height - 20) 

        ' Draw the thumb.
        If (m_ThumbLocation.Width > 0) AndAlso (m_ThumbLocation.Height > 0) Then
            Using foreground_pen As New Pen(Me.ForeColor)
                gr.DrawRectangle(foreground_pen, m_ThumbLocation)
            End Using
        End If
    End Sub

    Private Sub MapViewFinder_HandleCreated(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.HandleCreated
        Me.SetStyle( _
            ControlStyles.AllPaintingInWmPaint Or _
            ControlStyles.ResizeRedraw Or _
            ControlStyles.OptimizedDoubleBuffer, _
            True)
        Me.UpdateStyles()
    End Sub

    ' We have been resized. Make a new background image.
    Private Sub MapViewFinder_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        MakeTransformation()
    End Sub

    ' Make a transformation to display the map on our surface.
    Private Sub MakeTransformation()
        ' Do nothing if we have no map image.
        If m_MapImage Is Nothing Then Exit Sub
        If Me.Width < 1 Then Exit Sub
        If Me.Height < 1 Then Exit Sub

        ' Find the scales needed to shrink the map
        ' to fit vertically and horizontally.
        Dim scale_horz As Double = (Me.Width - 1) / m_MapImage.Width
        Dim scale_vert As Double = (Me.Height - 1) / m_MapImage.Height

        ' Use the smaller scale.
        Dim the_scale As Double = Math.Min(scale_horz, scale_vert)

        ' Make the transformation to map 
        ' the map image onto our surface.
        Dim scaled_map_wid As Double = m_MapImage.Width * the_scale
        Dim scaled_map_hgt As Double = m_MapImage.Height * the_scale
        Dim x0 As Double = (Me.Width - 1 - scaled_map_wid) / 2
        Dim y0 As Double = (Me.Height - 1 - scaled_map_hgt) / 2
        Dim pts() As PointF = { _
            New PointF(CSng(x0), CSng(y0)), _
            New PointF(CSng(x0 + scaled_map_wid), CSng(y0)), _
            New PointF(CSng(x0), CSng(y0 + scaled_map_hgt)) _
        }
        m_Transformation = New Matrix(m_MapImage.GetBounds(GraphicsUnit.Pixel), pts)

        ' Save the inverse transformation.
        m_InverseTransformation = m_Transformation.Clone()
        m_InverseTransformation.Invert()

        ' Redraw.
        Me.Invalidate()
    End Sub

#Region "Mouse Events"
    ' MouseDown. See if we are within the thumb.
    Private Sub MapViewFinder_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        ' Transform the mouse's position into map coordinates.
        Dim pts() As Point = {New Point(e.X, e.Y)}
        m_InverseTransformation.TransformPoints(pts)

        ' See if this is within the thumb.
        If Not m_ThumbLocation.Contains(pts(0)) Then Exit Sub

        ' It's inside the thumb. Start dragging.
        m_Dragging = True
        m_PrevX = pts(0).X
        m_PrevY = pts(0).Y
    End Sub

    ' Continue dragging.
    Private Sub MapViewFinder_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        ' See if we are dragging the thumb.
        If Not m_Dragging Then Exit Sub

        ' Transform the mouse's position into map coordinates.
        Dim pts() As Point = {New Point(e.X, e.Y)}
        m_InverseTransformation.TransformPoints(pts)

        ' Get the thumb's new position.
        Dim new_x As Integer = m_ThumbLocation.X + pts(0).X - m_PrevX
        Dim new_y As Integer = m_ThumbLocation.Y + pts(0).Y - m_PrevY

        ' Save the new position.
        m_PrevX = pts(0).X
        m_PrevY = pts(0).Y

        ' Make sure the thumb stays within the map's bounds.
        If new_x < 0 Then new_x = 0
        If new_y < 0 Then new_y = 0
        If new_x > m_MapImage.Width - m_ThumbLocation.Width - 1 Then
            new_x = m_MapImage.Width - m_ThumbLocation.Width
        End If
        If new_y > m_MapImage.Height - m_ThumbLocation.Height - 1 Then
            new_y = m_MapImage.Height - m_ThumbLocation.Height - 1
        End If

        ' See if it moved.
        If (new_x = m_ThumbLocation.X) AndAlso (new_y = m_ThumbLocation.Y) Then Exit Sub

        ' Move the thumb.
        m_ThumbLocation.X = new_x
        m_ThumbLocation.Y = new_y

        ' Redraw.
        Me.Invalidate()

        ' Raise the ThumbScroll event.
        RaiseEvent ThumbScroll()
    End Sub

    ' Stop dragging.
    Private Sub MapViewFinder_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        ' See if we are dragging the thumb.
        If Not m_Dragging Then Exit Sub

        m_Dragging = False

        ' Raise the ThumbScrollComplete event.
        RaiseEvent ThumbScrollComplete()
    End Sub

#End Region ' Mouse Events

End Class
