Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.Drawing
Imports System.Windows.Forms

<ToolboxBitmap(GetType(RegexHighlightProvider), "tbxRegexHighlightProvider")> _
<ProvideProperty("HighlightPattern", GetType(TextBox))> _
<ProvideProperty("HighlightAllowBlank", GetType(TextBox))> _
<ProvideProperty("HighlightInvalidColor", GetType(TextBox))> _
Public Class RegexHighlightProvider
    Implements IExtenderProvider

    ' Store a control's regular expression pattern and error message.
    Private Const ALLOW_BLANK_DEFAULT As Boolean = True
    Private Shared INVALID_COLOR_DEFAULT As Color = Color.Yellow
    Private Class HighlightInfo
        Public Pattern As String = ""
        Public AllowBlank As Boolean = ALLOW_BLANK_DEFAULT
        Public InvalidColor As Color = INVALID_COLOR_DEFAULT
        Public ValidColor As Color = Color.White
        Public Function IsBlank() As Boolean
            Return (Pattern.Length = 0) AndAlso _
                   (AllowBlank = ALLOW_BLANK_DEFAULT) AndAlso _
                   (InvalidColor.ToArgb() = INVALID_COLOR_DEFAULT.ToArgb())
        End Function
    End Class

    ' Hold information about controls and their regular expressions.
    Private m_HighlightInfos As New Generic.Dictionary(Of TextBox, HighlightInfo)

    ' Extend only TextBoxes.
    Public Function CanExtend(ByVal extendee As Object) As Boolean Implements System.ComponentModel.IExtenderProvider.CanExtend
        Return (TypeOf extendee Is TextBox)
    End Function

#Region "HighlightPattern Property"
    ' Return a TextBox's HighlightPattern value.
    Public Function GetHighlightPattern(ByVal client_control As TextBox) As String
        If m_HighlightInfos.ContainsKey(client_control) Then
            Return m_HighlightInfos.Item(client_control).Pattern
        Else
            Return ""
        End If
    End Function

    ' Set a TextBox's HighlightPattern value.
    Public Sub SetHighlightPattern(ByVal client_control As TextBox, ByVal new_pattern As String)
        If new_pattern Is Nothing Then new_pattern = ""

        ' Get the client's HighlightInfo object or make a new one.
        Dim highlight_info As HighlightInfo = GetOrMakeHighlightInfo(client_control)

        ' Save the new pattern.
        highlight_info.Pattern = new_pattern

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, highlight_info)
    End Sub
#End Region ' HighlightPattern Property

#Region "HighlightAllowBlank Property"
    ' Return a TextBox's HighlightAllowBlank value.
    Public Function GetHighlightAllowBlank(ByVal client_control As TextBox) As Boolean
        If m_HighlightInfos.ContainsKey(client_control) Then
            Return m_HighlightInfos.Item(client_control).AllowBlank
        Else
            Return ALLOW_BLANK_DEFAULT
        End If
    End Function

    ' Set a TextBox's HighlightAllowBlank value.
    Public Sub SetHighlightAllowBlank(ByVal client_control As TextBox, ByVal new_AllowBlank As Boolean)
        ' Get the client's HighlightInfo object or make a new one.
        Dim highlight_info As HighlightInfo = GetOrMakeHighlightInfo(client_control)

        ' Save the new AllowBlank.
        highlight_info.AllowBlank = new_AllowBlank

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, highlight_info)
    End Sub
#End Region ' HighlightAllowBlank Property

#Region "HighlightInvalidColor Property"
    ' Return a TextBox's HighlightInvalidColor value.
    Public Function GetHighlightInvalidColor(ByVal client_control As TextBox) As Color
        If m_HighlightInfos.ContainsKey(client_control) Then
            Return m_HighlightInfos.Item(client_control).InvalidColor
        Else
            Return INVALID_COLOR_DEFAULT
        End If
    End Function

    ' Set a TextBox's HighlightInvalidColor value.
    Public Sub SetHighlightInvalidColor(ByVal client_control As TextBox, ByVal new_color As Color)
        ' Get the client's HighlightInfo object or make a new one.
        Dim highlight_info As HighlightInfo = GetOrMakeHighlightInfo(client_control)

        ' Save the new color.
        highlight_info.InvalidColor = new_color

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, highlight_info)
    End Sub
#End Region ' HighlightInvalidColor Property

    ' Return the client's HighlightInfo or make a new one.
    Private Function GetOrMakeHighlightInfo(ByVal client_control As TextBox) As HighlightInfo
        If m_HighlightInfos.ContainsKey(client_control) Then
            Return m_HighlightInfos.Item(client_control)
        Else
            Return New HighlightInfo
        End If
    End Function

    ' Add or remove the entry if necessary.
    Private Sub AddOrRemoveIfNecessary(ByVal client_control As TextBox, ByVal highlight_info As HighlightInfo)
        ' See if it's all blank.
        If highlight_info.IsBlank() Then
            ' Remove the TextBox from the expression dictionary.
            If m_HighlightInfos.ContainsKey(client_control) Then
                m_HighlightInfos.Remove(client_control)
                RemoveHandler client_control.TextChanged, AddressOf Client_TextChanged
            End If
        Else
            ' Add or update the control's expression.
            m_HighlightInfos.Item(client_control) = highlight_info
            AddHandler client_control.TextChanged, AddressOf Client_TextChanged
        End If
    End Sub

    ' A client's text changed. See if it matches its expression.
    Private Sub Client_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        ' Get the TextBox and its information.
        Dim text_box As TextBox = DirectCast(sender, TextBox)
        If Not m_HighlightInfos.ContainsKey(text_box) Then Exit Sub
        Dim highlight_info As HighlightInfo = m_HighlightInfos(text_box)

        ' Validate the expression.
        If highlight_info.AllowBlank AndAlso (text_box.Text.Length = 0) Then
            ' Blank is okay. Clear the highlight.
            text_box.BackColor = highlight_info.ValidColor
        ElseIf Regex.IsMatch(text_box.Text, highlight_info.Pattern, RegexOptions.IgnoreCase) Then
            ' It matches. Clear the highlight.
            text_box.BackColor = highlight_info.ValidColor
        Else
            ' It doesn't match. Highlight the TextBox.
            If text_box.BackColor <> highlight_info.InvalidColor Then
                highlight_info.ValidColor = text_box.BackColor
                text_box.BackColor = highlight_info.InvalidColor
            End If
        End If
    End Sub

End Class
