Imports System.Xml

' Represents an item in the XML document.
Public Class DocNode
    ' The XML document's node for this item.
    Public XmlDocNode As XmlNode

    ' The node's fully-qualified name.
    Public FullName As String

    ' The last part of the node's name.
    Public ShortName As String

    ' The node's type code:
    '   A       Assembly
    '   #ctor   Constructor
    '   T       Type
    '   F       Field
    '   P       Property
    '   M       Method
    '   E       Event
    Public TypeName As String
    Public TypeCode As String

    ' The node's scope code:
    '   Public
    '   Protected
    '   Private
    '   Unknown
    Public ScopeName As String

    ' The node's logical children.
    Public Children As New Collection

    ' Save the node's name and type code.
    Public Sub New(ByVal xml_node As XmlNode, ByVal new_FullName As String, ByVal new_TypeCode As String)
        ' Save the XML node.
        XmlDocNode = xml_node

        ' Save the full name.
        FullName = new_FullName

        ' Get the short name.
        ' Start searching before any (.
        Dim paren_pos As Integer = FullName.IndexOf("(")
        If paren_pos < 0 Then paren_pos = FullName.Length
        If FullName.LastIndexOf(".", paren_pos) >= 0 Then
            ShortName = FullName.Substring(FullName.LastIndexOf(".", paren_pos) + 1)
        Else
            ShortName = FullName
        End If

        ' Save the type and scope codes.
        TypeCode = new_TypeCode
        TypeName = TypeCodeToString(TypeCode)

        ' Get the scope code.
        Dim scope_node As XmlNode = Nothing
        If xml_node IsNot Nothing Then scope_node = xml_node.Item("scope")
        If scope_node Is Nothing Then
            ScopeName = ""
        Else
            ScopeName = scope_node.InnerText
        End If
    End Sub

    ' Add a child to this node's Children collection.
    Public Sub AddChild(ByVal new_child As DocNode)
        ' See if the new child has our full name.
        If new_child.FullName = Me.FullName Then
            ' The child has information that belongs in our node.
            ' Save the XML node and type code, and discard the new child.
            Me.XmlDocNode = new_child.XmlDocNode
            Me.TypeCode = new_child.TypeCode
            Me.TypeName = new_child.TypeName
            Me.ScopeName = new_child.ScopeName
        Else
            ' Get the next piece of the child's name.
            ' Remove the first part.
            Dim name_piece As String = new_child.FullName.Substring(Me.FullName.Length + 1)
            ' Remove the last part.
            Dim paren_pos As Integer = name_piece.IndexOf("(")
            If paren_pos < 0 Then paren_pos = name_piece.Length
            If name_piece.LastIndexOf(".", paren_pos) >= 0 Then
                name_piece = name_piece.Substring(0, name_piece.IndexOf("."))
            End If

            ' See if we have a child with this short name.
            If Children.Contains(name_piece) Then
                ' We have this child. Get it.
                Dim child_node As DocNode = DirectCast(Children.Item(name_piece), DocNode)

                ' Put the new child inside this child.
                child_node.AddChild(new_child)
            Else
                ' We don't have the needed child. Make it.
                Dim child_node As New DocNode(Nothing, Me.FullName & "." & name_piece, "?")
                Me.Children.Add(child_node, name_piece)

                ' Put the new child inside this child.
                child_node.AddChild(new_child)
            End If
        End If
    End Sub

    ' Display the node's data.
    Public Function PrefixOrder(Optional ByVal indent As Integer = 0) As String
        Dim result As String = Space(indent) & ShortDescription & vbCrLf

        For Each child_node As DocNode In Children
            result &= child_node.PrefixOrder(indent + 4)
        Next child_node
        Return result
    End Function

    ' Return a short description for this object.
    Private Function ShortDescription() As String
        Return _
            Me.ScopeName & " " & _
            Me.TypeName & ":" & _
            Me.FullName
    End Function

    ' Return a string describing the type code.
    Private Function TypeCodeToString(ByVal type_code As String) As String
        Select Case type_code
            Case "A"
                Return "Assembly"
            Case "#ctor"
                Return "Constructor"
            Case "T"
                Return "Type"
            Case "F"
                Return "Field"
            Case "P"
                Return "Property"
            Case "M"
                Return "Method"
            Case "E"
                Return "Event"
            Case Else
                Return "<Unknown>"
        End Select
    End Function

    ' Sort the node's children in these groups:
    '   A       Assembly
    '   #ctor   Constructor
    '   T       Type
    '   F       Field
    '   P       Property
    '   M       Method
    '   E       Event
    ' And alphabetically within each group.
    Public Sub SortChildren()
        ' Copy the children into a temporary array.
        Dim new_children(0 To Children.Count - 1) As DocNode
        For i As Integer = 0 To Children.Count - 1
            new_children(i) = DirectCast(Children.Item(i + 1), DocNode)
        Next i

        ' Sort the array.
        Dim doc_node_comparer As New DocNodeComparer()
        Array.Sort(Of DocNode)(new_children, doc_node_comparer)

        ' Copy the sorted children back into the Children collection.
        Children = New Collection
        For i As Integer = 0 To new_children.Length - 1
            Children.Add(new_children(i), new_children(i).ShortName)
        Next i

        ' Recursively sort the children.
        For Each child_node As DocNode In Children
            child_node.SortChildren()
        Next child_node
    End Sub

    Private Class DocNodeComparer
        Implements IComparer(Of DocNode)

        ' Determines which type codes come first.
        Private m_TypeCodeRanks As New Collection

        ' Determines which scope come first.
        Private m_ScopeRanks As New Collection

        ' Initialize the type code ranks.
        Public Sub New()
            m_TypeCodeRanks.Add("A-", "A")
            m_TypeCodeRanks.Add("B-", "#ctor")
            m_TypeCodeRanks.Add("C-", "T")
            m_TypeCodeRanks.Add("D-", "F")
            m_TypeCodeRanks.Add("E-", "P")
            m_TypeCodeRanks.Add("F-", "M")
            m_TypeCodeRanks.Add("G-", "E")
            m_TypeCodeRanks.Add("Z-", "?")

            m_ScopeRanks.Add("A-", "Public")
            m_ScopeRanks.Add("B-", "Protected")
            m_ScopeRanks.Add("C-", "Private")
            m_ScopeRanks.Add("Z-", "?")
        End Sub
        Public Function TypeCodeRank(ByVal type_code As String) As String
            If m_TypeCodeRanks.Contains(type_code) Then Return CStr(m_TypeCodeRanks(type_code))
            Return CStr(m_TypeCodeRanks("?"))
        End Function
        Public Function ScopeNameRank(ByVal scope_code As String) As String
            If m_ScopeRanks.Contains(scope_code) Then Return CStr(m_ScopeRanks(scope_code))
            Return CStr(m_ScopeRanks("?"))
        End Function

        ' Compare two DocNodes.
        Public Function Compare(ByVal x As DocNode, ByVal y As DocNode) As Integer Implements System.Collections.Generic.IComparer(Of DocNode).Compare
            Dim x_name As String = _
                ScopeNameRank(x.TypeCode) & _
                TypeCodeRank(x.TypeCode) & _
                x.ShortName
            Dim y_name As String = _
                ScopeNameRank(y.TypeCode) & _
                TypeCodeRank(y.TypeCode) & _
                y.ShortName
            Return String.Compare(x_name, y_name)
        End Function
    End Class

End Class
